@@ -167,6 +167,16 @@ def click_global_conf_with_output_format(f):
     return f
 
 
+def click_option_metadata(f):
+    """Add metadata option to command."""
+    return click.option(
+        '--metadata-path',
+        'metadata',
+        callback=load_json_from_path,
+        type=click.Path(exists=True, resolve_path=True, dir_okay=False),
+        help='Metadata file path')(f)
+
+
 def click_option_expand(f):
     """Add expand option to command."""
     return click.option(
@@ -191,7 +201,7 @@ def load_data_samples_keys(data_samples, option="--data-samples-path"):
     try:
         return data_samples['keys']
     except KeyError:
-        raise click.BadParameter(f'File must contain a "keys" attribute.', param_hint=f'"{option}"')
+        raise click.BadParameter('File must contain a "keys" attribute.', param_hint=f'"{option}"')
 
 
 def error_printer(fn):
@@ -208,6 +218,20 @@ def error_printer(fn):
         except exceptions.BadLoginException:
             raise click.ClickException('Login failed: No active account found with the'
                                        ' given credentials.')
+        except exceptions.InvalidRequest as e:
+            if not isinstance(e.errors, dict):
+                raise click.ClickException(f"Request failed: {e.__class__.__name__}: {e}")
+
+            lines = []
+            for field, errors in e.errors.items():
+                for error in errors:
+                    lines.append(f"- {field}: {error}")
+
+            action = fn.__name__.replace('_', ' ')
+            pluralized_error = 'errors' if len(lines) > 1 else 'error'
+            lines.insert(0, f"Could not {action},"
+                            f"the server returned the following {pluralized_error}:")
+            raise click.ClickException("\n".join(lines))
         except exceptions.RequestException as e:
             raise click.ClickException(f"Request failed: {e.__class__.__name__}: {e}")
         except (exceptions.ConnectionError,
@@ -241,14 +265,10 @@ def cli(ctx):
 @click.option('--insecure', '-k', is_flag=True,
               help='Do not verify SSL certificates')
 @click.option('--version', '-v', default=configuration.DEFAULT_VERSION)
-@click.option('--username', '-u', required=True)
-@click.option('--password', '-p', required=True)
-def add_profile_to_config(url, config, profile, insecure, version, username, password):
+def add_profile_to_config(url, config, profile, insecure, version):
     """Add profile to config file."""
     configuration.Manager(config).add_profile(
         profile,
-        username,
-        password,
         url,
         version=version,
         insecure=insecure,
@@ -259,14 +279,17 @@ def add_profile_to_config(url, config, profile, insecure, version, username, pas
 @click_global_conf
 @click.pass_context
 @error_printer
-def login(ctx):
+@click.option('--username', '-u', envvar='SUBSTRA_USERNAME', prompt=True)
+@click.option('--password', '-p', envvar='SUBSTRA_PASSWORD', prompt=True, hide_input=True)
+def login(ctx, username, password):
     """Login to the Substra platform."""
     usr.Manager(ctx.obj.user).clear_user()
     client = get_client(ctx.obj)
 
-    token = client.login()
+    token = client.login(username, password)
     # create temporary user data
     usr.Manager(ctx.obj.user).add_user(token)
+    display(f"Token: {token}")
 
 
 @cli.group()
@@ -339,6 +362,7 @@ def add_dataset(ctx, data, objective_key):
             "public": bool,
             "authorized_ids": list[str],
         },
+        "metadata": dict
     }
 
     \b
@@ -354,6 +378,7 @@ def add_dataset(ctx, data, objective_key):
     client = get_client(ctx.obj)
     if objective_key:  # overwrite data values if set
         data['objective_key'] = objective_key
+
     res = client.add_dataset(data)
     printer = printers.get_asset_printer(assets.DATASET, ctx.obj.output_format)
     printer.print(res, is_list=False)
@@ -384,6 +409,7 @@ def add_objective(ctx, data, dataset_key, data_samples):
             "public": bool,
             "authorized_ids": list[str],
         },
+        "metadata": dict
     }
 
     \b
@@ -440,6 +466,7 @@ def add_algo(ctx, data):
             "public": bool,
             "authorized_ids": list[str],
         },
+        "metadata": dict
     }
 
     \b
@@ -450,6 +477,7 @@ def add_algo(ctx, data):
       script and its Dockerfile
     - permissions: define asset access permissions
     """
+
     client = get_client(ctx.obj)
     res = client.add_algo(data)
     printer = printers.get_asset_printer(assets.ALGO, ctx.obj.output_format)
@@ -457,15 +485,15 @@ def add_algo(ctx, data):
 
 
 @add.command('compute_plan')
-@click.argument('tuples', type=click.Path(exists=True, dir_okay=False),
-                callback=load_json_from_path, metavar="TUPLES_PATH")
+@click.argument('data', type=click.Path(exists=True, dir_okay=False),
+                callback=load_json_from_path, metavar="PATH")
 @click_global_conf_with_output_format
 @click.pass_context
 @error_printer
-def add_compute_plan(ctx, tuples):
+def add_compute_plan(ctx, data):
     """Add compute plan.
 
-    The tuples path must point to a valid JSON file with the following schema:
+    The path must point to a valid JSON file with the following schema:
 
     \b
     {
@@ -476,6 +504,7 @@ def add_compute_plan(ctx, tuples):
             "traintuple_id": str,
             "in_models_ids": list[str],
             "tag": str,
+            "metadata": dict
         }],
         "composite_traintuples": list[{
             "composite_traintuple_id": str,
@@ -488,6 +517,7 @@ def add_compute_plan(ctx, tuples):
                 "authorized_ids": list[str],
             },
             "tag": str,
+            "metadata": dict
         }]
         "aggregatetuples": list[{
             "aggregatetuple_id": str,
@@ -495,6 +525,7 @@ def add_compute_plan(ctx, tuples):
             "worker": str,
             "in_models_ids": list[str],
             "tag": str,
+            "metadata": dict
         }],
         "testtuples": list[{
             "objective_key": str,
@@ -502,12 +533,16 @@ def add_compute_plan(ctx, tuples):
             "test_data_sample_keys": list[str],
             "traintuple_id": str,
             "tag": str,
-        }]
+            "metadata": dict
+        }],
+        "clean_models": bool,
+        "tag": str,
+        "metadata": dict
     }
 
     """
     client = get_client(ctx.obj)
-    res = client.add_compute_plan(tuples)
+    res = client.add_compute_plan(data)
     printer = printers.get_asset_printer(assets.COMPUTE_PLAN, ctx.obj.output_format)
     printer.print(res, is_list=False)
 
@@ -532,6 +567,7 @@ def add_aggregate_algo(ctx, data):
             "public": bool,
             "authorized_ids": list[str],
         },
+        "metadata": dict
     }
 
     \b
@@ -542,6 +578,7 @@ def add_aggregate_algo(ctx, data):
       script and its Dockerfile
     - permissions: define asset access permissions
     """
+
     client = get_client(ctx.obj)
     res = client.add_aggregate_algo(data)
     printer = printers.get_asset_printer(assets.AGGREGATE_ALGO, ctx.obj.output_format)
@@ -568,6 +605,7 @@ def add_composite_algo(ctx, data):
             "public": bool,
             "authorized_ids": list[str],
         },
+        "metadata": dict
     }
 
     \b
@@ -578,6 +616,7 @@ def add_composite_algo(ctx, data):
       script and its Dockerfile
     - permissions: define asset access permissions
     """
+
     client = get_client(ctx.obj)
     res = client.add_composite_algo(data)
     printer = printers.get_asset_printer(assets.COMPOSITE_ALGO, ctx.obj.output_format)
@@ -594,9 +633,10 @@ def add_composite_algo(ctx, data):
               help='In model traintuple key.')
 @click.option('--tag')
 @click_global_conf_with_output_format
+@click_option_metadata
 @click.pass_context
 @error_printer
-def add_traintuple(ctx, algo_key, dataset_key, data_samples, in_models_keys, tag):
+def add_traintuple(ctx, algo_key, dataset_key, data_samples, in_models_keys, tag, metadata):
     """Add traintuple.
 
     The option --data-samples-path must point to a valid JSON file with the
@@ -623,6 +663,9 @@ def add_traintuple(ctx, algo_key, dataset_key, data_samples, in_models_keys, tag
     if tag:
         data['tag'] = tag
 
+    if metadata:
+        data['metadata'] = metadata
+
     if in_models_keys:
         data['in_models_keys'] = in_models_keys
     res = client.add_traintuple(data)
@@ -638,9 +681,10 @@ def add_traintuple(ctx, algo_key, dataset_key, data_samples, in_models_keys, tag
 @click.option('--rank', type=click.INT)
 @click.option('--tag')
 @click_global_conf_with_output_format
+@click_option_metadata
 @click.pass_context
 @error_printer
-def add_aggregatetuple(ctx, algo_key, in_models_keys, worker, rank, tag):
+def add_aggregatetuple(ctx, algo_key, in_models_keys, worker, rank, tag, metadata):
     """Add aggregatetuple."""
     client = get_client(ctx.obj)
     data = {
@@ -656,6 +700,9 @@ def add_aggregatetuple(ctx, algo_key, in_models_keys, worker, rank, tag):
 
     if tag:
         data['tag'] = tag
+
+    if metadata:
+        data['metadata'] = metadata
     res = client.add_aggregatetuple(data)
     printer = printers.get_asset_printer(assets.AGGREGATETUPLE, ctx.obj.output_format)
     printer.print(res, is_list=False)
@@ -677,10 +724,11 @@ def add_aggregatetuple(ctx, algo_key, in_models_keys, worker, rank, tag):
               help='Load a permissions file.')
 @click.option('--tag')
 @click_global_conf_with_output_format
+@click_option_metadata
 @click.pass_context
 @error_printer
 def add_composite_traintuple(ctx, algo_key, dataset_key, data_samples, head_model_key,
-                             trunk_model_key, out_trunk_model_permissions, tag):
+                             trunk_model_key, out_trunk_model_permissions, tag, metadata):
     """Add composite traintuple.
 
     The option --data-samples-path must point to a valid JSON file with the
@@ -729,6 +777,9 @@ def add_composite_traintuple(ctx, algo_key, dataset_key, data_samples, head_mode
 
     if tag:
         data['tag'] = tag
+
+    if metadata:
+        data['metadata'] = metadata
     res = client.add_composite_traintuple(data)
     printer = printers.get_asset_printer(assets.COMPOSITE_TRAINTUPLE, ctx.obj.output_format)
     printer.print(res, is_list=False)
@@ -743,9 +794,10 @@ def add_composite_traintuple(ctx, algo_key, dataset_key, data_samples, head_mode
               callback=load_json_from_path)
 @click.option('--tag')
 @click_global_conf_with_output_format
+@click_option_metadata
 @click.pass_context
 @error_printer
-def add_testtuple(ctx, objective_key, dataset_key, traintuple_key, data_samples, tag):
+def add_testtuple(ctx, objective_key, dataset_key, traintuple_key, data_samples, tag, metadata):
     """Add testtuple.
 
     The option --data-samples-path must point to a valid JSON file with the
@@ -772,6 +824,9 @@ def add_testtuple(ctx, objective_key, dataset_key, traintuple_key, data_samples,
 
     if tag:
         data['tag'] = tag
+
+    if metadata:
+        data['metadata'] = metadata
     res = client.add_testtuple(data)
     printer = printers.get_asset_printer(assets.TESTTUPLE, ctx.obj.output_format)
     printer.print(res, is_list=False)
@@ -841,18 +896,10 @@ def get(ctx, expand, asset_name, asset_key):
               callback=validate_json,
               help='Filter results using a complex search (must be a JSON array of valid filters). '
                    'Incompatible with the --filter option')
-@click.option(
-    '--is-complex', is_flag=True,
-    help=(
-        "When using filters using 'OR', the server will return a list of matching assets for each "
-        "operand. By default these lists are merged into a single list. When set, this option "
-        "disables the lists aggregation."
-    ),
-)
 @click_global_conf_with_output_format
 @click.pass_context
 @error_printer
-def list_(ctx, asset_name, filters, filters_logical_clause, advanced_filters, is_complex):
+def list_(ctx, asset_name, filters, filters_logical_clause, advanced_filters):
     """List assets."""
     client = get_client(ctx.obj)
     # method must exist in sdk
@@ -869,7 +916,7 @@ def list_(ctx, asset_name, filters, filters_logical_clause, advanced_filters, is
                 filters.insert(i + 1, 'OR')
     elif advanced_filters:
         filters = advanced_filters
-    res = method(filters, is_complex)
+    res = method(filters)
     printer = printers.get_asset_printer(asset_name, ctx.obj.output_format)
     printer.print(res, is_list=True)
 
@@ -1006,7 +1053,7 @@ def run_local(algo, train_opener, test_opener, metrics, rank,
     if not fake_data_samples and not train_data_samples and not test_data_samples:
         raise click.BadOptionUsage('--fake-data-samples',
                                    'Missing option --fake-data-samples or --test-data-samples '
-                                   'and -train-data-samples')
+                                   'and --train-data-samples')
     if not fake_data_samples and train_data_samples and not test_data_samples:
         raise click.BadOptionUsage('--test-data-samples',
                                    'Missing option --test-data-samples')
@@ -1120,6 +1167,7 @@ def update_compute_plan(ctx, compute_plan_id, tuples):
             "traintuple_id": str,
             "in_models_ids": list[str],
             "tag": str,
+            "metadata": dict,
         }],
         "composite_traintuples": list[{
             "composite_traintuple_id": str,
@@ -1132,6 +1180,7 @@ def update_compute_plan(ctx, compute_plan_id, tuples):
                 "authorized_ids": list[str],
             },
             "tag": str,
+            "metadata": dict,
         }]
         "aggregatetuples": list[{
             "aggregatetuple_id": str,
@@ -1139,6 +1188,7 @@ def update_compute_plan(ctx, compute_plan_id, tuples):
             "worker": str,
             "in_models_ids": list[str],
             "tag": str,
+            "metadata": dict,
         }],
         "testtuples": list[{
             "objective_key": str,
@@ -1146,6 +1196,7 @@ def update_compute_plan(ctx, compute_plan_id, tuples):
             "test_data_sample_keys": list[str],
             "traintuple_id": str,
             "tag": str,
+            "metadata": dict,
         }]
     }
 
