@@ -17,8 +17,7 @@ import functools
 import logging
 import os
 import time
-
-import keyring
+import json
 
 from substra.sdk import utils, assets, rest_client, exceptions
 from substra.sdk import config as cfg
@@ -53,35 +52,10 @@ def get_asset_key(data):
     return data.get('pkhash') or data.get('key')
 
 
-def _flatten_permissions(data, field_name):
-    data = deepcopy(data)
-
-    p = data[field_name]  # should not raise as a default permissions is added if missing
-    data[f'{field_name}_public'] = p['public']
-    data[f'{field_name}_authorized_ids'] = p.get('authorized_ids', [])
-    del data[field_name]
-
-    return data
-
-
-def _update_permissions_field(data, permissions_field='permissions'):
-    if permissions_field not in data:
-        default_permissions = {
-            'public': False,
-            'authorized_ids': [],
-        }
-        data[permissions_field] = default_permissions
-    # XXX workaround because backend accepts only Form Data body. This is due to
-    #     the fact that backend expects both file objects and payload in the
-    #     same request
-    data = _flatten_permissions(data, field_name=permissions_field)
-    return data
-
-
 class Client(object):
 
     def __init__(self, config_path=None, profile_name=None, user_path=None,
-                 retry_timeout=DEFAULT_RETRY_TIMEOUT):
+                 token=None, retry_timeout=DEFAULT_RETRY_TIMEOUT):
         self._cfg_manager = cfg.Manager(config_path or cfg.DEFAULT_PATH)
         self._usr_manager = usr.Manager(user_path or usr.DEFAULT_PATH)
         self._current_profile = None
@@ -97,24 +71,25 @@ class Client(object):
         # set current logged user if exists
         self.set_user()
 
+        if token:
+            self._set_token(token)
+
     @logit
-    def login(self):
-        """Login.
-
-        Allow to login to a remote server.
-
-        After setting your configuration with `substra config` using `-u` and `-p`
-        Launch `substra login`
-        You will get a token which will be stored by default in `~/.substra-user`
-        You can change that thanks to the --user option (works like the --profile option)
-
-        """
+    def login(self, username, password):
+        """Login to a remote server. """
 
         if not self._current_profile:
             raise exceptions.SDKException("No profile defined")
 
-        res = self.client.login()
+        res = self.client.login(username, password)
         token = res.json()['token']
+        self._set_token(token)
+        return token
+
+    def _set_token(self, token):
+        if not self._current_profile:
+            raise exceptions.SDKException("No profile defined")
+
         self._current_profile.update({
             'token': token,
         })
@@ -155,28 +130,29 @@ class Client(object):
                 })
                 self.client.set_config(self._current_profile, self._profile_name)
 
-    def add_profile(self, profile_name, username, password, url, version='0.0', insecure=False):
+    def add_profile(self, profile_name, url, version='0.0', insecure=False):
         """Add new profile (in-memory only)."""
         profile = cfg.create_profile(
             url=url,
             version=version,
             insecure=insecure,
-            username=username,
         )
-        keyring.set_password(profile_name, username, password)
         return self._set_current_profile(profile_name, profile)
 
-    def _add(self, asset, data, files=None, exist_ok=False, json_encoding=False):
+    def _add(self, asset, data, files=None, exist_ok=False):
         """Add asset."""
         data = deepcopy(data)  # make a deep copy for avoiding modification by reference
-        files = files or {}
-        requests_kwargs = {}
         if files:
-            requests_kwargs['files'] = files
-        if json_encoding:
-            requests_kwargs['json'] = data
+            requests_kwargs = {
+                'data': {
+                    'json': json.dumps(data),
+                },
+                'files': files,
+            }
         else:
-            requests_kwargs['data'] = data
+            requests_kwargs = {
+                'json': data
+            }
 
         return self.client.add(
             asset,
@@ -227,7 +203,6 @@ class Client(object):
         existing asset will be returned.
 
         """
-        data = _update_permissions_field(data)
         if 'paths' in data:
             raise ValueError("data: invalid 'paths' field")
         if 'path' not in data:
@@ -273,7 +248,6 @@ class Client(object):
         If data samples with the same content as any of the paths already exists, an `AlreadyExists`
         exception will be raised.
         """
-        data = _update_permissions_field(data)
         if 'path' in data:
             raise ValueError("data: invalid 'path' field")
         if 'paths' not in data:
@@ -297,6 +271,7 @@ class Client(object):
                 "public": bool,
                 "authorized_ids": list[str],
             },
+            "metadata": dict
         }
 ```
 
@@ -306,7 +281,6 @@ class Client(object):
         If `exist_ok` is true, `AlreadyExists` exceptions will be ignored and the
         existing asset will be returned.
         """
-        data = _update_permissions_field(data)
         attributes = ['data_opener', 'description']
         with utils.extract_files(data, attributes) as (data, files):
             res = self._add(assets.DATASET, data, files=files, exist_ok=exist_ok)
@@ -333,6 +307,7 @@ class Client(object):
                 "public": bool,
                 "authorized_ids": list[str],
             },
+            "metadata": dict
         }
 ```
 
@@ -342,7 +317,6 @@ class Client(object):
         If `exist_ok` is true, `AlreadyExists` exceptions will be ignored and the
         existing asset will be returned.
         """
-        data = _update_permissions_field(data)
         attributes = ['metrics', 'description']
         with utils.extract_files(data, attributes) as (data, files):
             res = self._add(assets.OBJECTIVE, data, files=files, exist_ok=exist_ok)
@@ -366,6 +340,7 @@ class Client(object):
                 "public": bool,
                 "authorized_ids": list[str],
             },
+            "metadata": dict
         }
 ```
 
@@ -375,7 +350,6 @@ class Client(object):
         If `exist_ok` is true, `AlreadyExists` exceptions will be ignored and the
         existing asset will be returned.
         """
-        data = _update_permissions_field(data)
         attributes = ['file', 'description']
         with utils.extract_files(data, attributes) as (data, files):
             res = self._add(assets.ALGO, data, files=files, exist_ok=exist_ok)
@@ -397,6 +371,7 @@ class Client(object):
                 "public": bool,
                 "authorizedIDs": list[str],
             },
+            "metadata": dict
         }
 ```
         If an aggregate algo with the same archive file already exists, an `AlreadyExists`
@@ -405,7 +380,6 @@ class Client(object):
         If `exist_ok` is true, `AlreadyExists` exceptions will be ignored and the
         existing asset will be returned.
         """
-        data = _update_permissions_field(data)
         attributes = ['file', 'description']
         with utils.extract_files(data, attributes) as (data, files):
             res = self._add(assets.AGGREGATE_ALGO, data, files=files, exist_ok=exist_ok)
@@ -427,6 +401,7 @@ class Client(object):
                 "public": bool,
                 "authorized_ids": list[str],
             },
+            "metadata": dict
         }
 ```
         If a composite algo with the same archive file already exists, an `AlreadyExists` exception
@@ -435,7 +410,6 @@ class Client(object):
         If `exist_ok` is true, `AlreadyExists` exceptions will be ignored and the
         existing asset will be returned.
         """
-        data = _update_permissions_field(data)
         attributes = ['file', 'description']
         with utils.extract_files(data, attributes) as (data, files):
             res = self._add(assets.COMPOSITE_ALGO, data, files=files, exist_ok=exist_ok)
@@ -457,6 +431,7 @@ class Client(object):
             "train_data_sample_keys": list[str],
             "in_models_keys": list[str],
             "tag": str,
+            "metadata": dict,
             "rank": int,
             "compute_plan_id": str,
         }
@@ -483,6 +458,7 @@ class Client(object):
             "algo_key": str,
             "in_models_keys": list[str],
             "tag": str,
+            "metadata": dict,
             "compute_plan_id": str,
             "rank": int,
             "worker": str,
@@ -515,6 +491,7 @@ class Client(object):
                 "authorized_ids": list[str],
             },
             "tag": str,
+            "metadata": dict,
             "rank": int,
             "compute_plan_id": str,
         }
@@ -531,7 +508,6 @@ class Client(object):
         If `exist_ok` is true, `AlreadyExists` exceptions will be ignored and the
         existing asset will be returned.
         """
-        data = _update_permissions_field(data, permissions_field='out_trunk_model_permissions')
         res = self._add(assets.COMPOSITE_TRAINTUPLE, data, exist_ok=exist_ok)
 
         # The backend has inconsistent API responses when getting or adding an asset (with much
@@ -551,10 +527,11 @@ class Client(object):
             "traintuple_key": str,
             "test_data_sample_keys": list[str],
             "tag": str,
+            "metadata": dict
         }
 ```
 
-        An `AlreadyExists` exception will be raised if a traintuple already exists that:
+        An `AlreadyExists` exception will be raised if a testtuple already exists that:
         * has the same `traintuple_key`, `objective_key`, `data_manager_key` and
           `test_data_sample_keys`
         * and was created through the same node you are using
@@ -583,6 +560,7 @@ class Client(object):
                 "train_data_sample_keys": list[str],
                 "in_models_ids": list[str],
                 "tag": str,
+                "metadata": dict,
             }],
             "composite_traintuples": list[{
                 "composite_traintuple_id": str,
@@ -595,6 +573,7 @@ class Client(object):
                     "authorized_ids": list[str],
                 },
                 "tag": str,
+                "metadata": dict,
             }]
             "aggregatetuples": list[{
                 "aggregatetuple_id": str,
@@ -602,6 +581,7 @@ class Client(object):
                 "worker": str,
                 "in_models_ids": list[str],
                 "tag": str,
+                "metadata": dict,
             }],
             "testtuples": list[{
                 "objective_key": str,
@@ -609,16 +589,18 @@ class Client(object):
                 "test_data_sample_keys": list[str],
                 "traintuple_id": str,
                 "tag": str,
+                "metadata": dict,
             }],
             "clean_models": bool,
-            "tag": str
+            "tag": str,
+            "metadata": dict
         }
 ```
 
         As specified in the data dict structure, output trunk models of composite
         traintuples cannot be made public.
         """
-        return self._add(assets.COMPUTE_PLAN, data, json_encoding=True)
+        return self._add(assets.COMPUTE_PLAN, data)
 
     @logit
     def get_algo(self, algo_key):
@@ -671,57 +653,57 @@ class Client(object):
         return self.client.get(assets.COMPOSITE_TRAINTUPLE, composite_traintuple_key)
 
     @logit
-    def list_algo(self, filters=None, is_complex=False):
+    def list_algo(self, filters=None):
         """List algos."""
         return self.client.list(assets.ALGO, filters=filters)
 
     @logit
-    def list_compute_plan(self, filters=None, is_complex=False):
+    def list_compute_plan(self, filters=None):
         """List compute plans."""
         return self.client.list(assets.COMPUTE_PLAN, filters=filters)
 
     @logit
-    def list_aggregate_algo(self, filters=None, is_complex=False):
+    def list_aggregate_algo(self, filters=None):
         """List aggregate algos."""
         return self.client.list(assets.AGGREGATE_ALGO, filters=filters)
 
     @logit
-    def list_composite_algo(self, filters=None, is_complex=False):
+    def list_composite_algo(self, filters=None):
         """List composite algos."""
         return self.client.list(assets.COMPOSITE_ALGO, filters=filters)
 
     @logit
-    def list_data_sample(self, filters=None, is_complex=False):
+    def list_data_sample(self, filters=None):
         """List data samples."""
         return self.client.list(assets.DATA_SAMPLE, filters=filters)
 
     @logit
-    def list_dataset(self, filters=None, is_complex=False):
+    def list_dataset(self, filters=None):
         """List datasets."""
         return self.client.list(assets.DATASET, filters=filters)
 
     @logit
-    def list_objective(self, filters=None, is_complex=False):
+    def list_objective(self, filters=None):
         """List objectives."""
         return self.client.list(assets.OBJECTIVE, filters=filters)
 
     @logit
-    def list_testtuple(self, filters=None, is_complex=False):
+    def list_testtuple(self, filters=None):
         """List testtuples."""
         return self.client.list(assets.TESTTUPLE, filters=filters)
 
     @logit
-    def list_traintuple(self, filters=None, is_complex=False):
+    def list_traintuple(self, filters=None):
         """List traintuples."""
         return self.client.list(assets.TRAINTUPLE, filters=filters)
 
     @logit
-    def list_aggregatetuple(self, filters=None, is_complex=False):
+    def list_aggregatetuple(self, filters=None):
         """List aggregatetuples."""
         return self.client.list(assets.AGGREGATETUPLE, filters=filters)
 
     @logit
-    def list_composite_traintuple(self, filters=None, is_complex=False):
+    def list_composite_traintuple(self, filters=None):
         """List composite traintuples."""
         return self.client.list(assets.COMPOSITE_TRAINTUPLE, filters=filters)
 
@@ -755,6 +737,7 @@ class Client(object):
                 "train_data_sample_keys": list[str],
                 "in_models_ids": list[str],
                 "tag": str,
+                "metadata": dict,
             }],
             "composite_traintuples": list[{
                 "composite_traintuple_id": str,
@@ -767,6 +750,7 @@ class Client(object):
                     "authorized_ids": list[str],
                 },
                 "tag": str,
+                "metadata": dict,
             }]
             "aggregatetuples": list[{
                 "aggregatetuple_id": str,
@@ -774,6 +758,7 @@ class Client(object):
                 "worker": str,
                 "in_models_ids": list[str],
                 "tag": str,
+                "metadata": dict,
             }],
             "testtuples": list[{
                 "objective_key": str,
@@ -781,6 +766,7 @@ class Client(object):
                 "test_data_sample_keys": list[str],
                 "traintuple_id": str,
                 "tag": str,
+                "metadata": dict,
             }]
         }
 ```
