@@ -63,14 +63,24 @@ class GatewayUnavailable(HTTPError):
 
 
 class InvalidRequest(HTTPError):
+    def __init__(self, msg, status_code, errors=None):
+        super().__init__(msg, status_code)
+        self.errors = errors
+
     @classmethod
     def from_request_exception(cls, request_exception):
         try:
             error = request_exception.response.json()
         except ValueError:
             error = request_exception.response
-        msg = error.get('message', None)
-        return super().from_request_exception(request_exception, msg)
+        msg = error.get('message', str(error))
+
+        try:
+            status_code = request_exception.response.status_code
+        except AttributeError:
+            status_code = None
+
+        return cls(msg, status_code, error)
 
 
 class NotFound(HTTPError):
@@ -145,8 +155,3 @@ class BadConfiguration(SDKException):
 class UserException(SDKException):
     """User Exception"""
     pass
-
-
-class KeyringException(SDKException):
-    """Could not retrieve password from keyring"""
-    pass
